part of 'sales_bloc.dart';

abstract class SalesState extends Equatable {
  const SalesState();

  @override
  List<Object> get props => [];
}

class SalesInitial extends SalesState {
  const SalesInitial();
}

class SalesLoading extends SalesState {
  const SalesLoading();
}

class SalesReady extends SalesState {
  const SalesReady();
}

class SalesSuccessful extends SalesState {
  const SalesSuccessful();
}

class SalesError extends SalesState {
  final String message;
  const SalesError({required this.message});

  @override
  List<Object> get props => [message];
}
